/* $Id: usb_rw.c,v 1.1 2009/02/20 02:20:31 jrothwei Exp $ */
/* Joseph Rothweiler, Sensicomm LLC. Started 19Feb2009.
** http://www.sensicomm.com
** Inspired by the blog post on 6sep2008 at http://braiden.org/?p=59
**
** Copyright 2009 Joseph Rothweiler
**
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
** The objective is to exercise the USB interface on
** the Nexys/Nexys2 FPGA boards (www.digilentinc.com).
**
** To use, first download the USB chip's program. See the
** associated directory for details.
**
** Also, burn the data handling program into the FPGA
** memory.
**
** The Cypress USB chip is operated in essentially its
** default mode:
** Endpoints EP2 and EP4 are outputs from the host PC,
** and EP6 and EP8 are inputs.
**
** EP2 and EP6 have the biggest buffers, so they will be
** used for the main transfers.
*/

#include <usb.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

int main( int argc, const char* argv[] ) {
	struct usb_bus *bus;
	struct usb_device *fx2 = NULL;
	int numfound;

	/* Initialize libusb. */
	usb_init();
	usb_set_debug( 1 );
	usb_find_busses();
	usb_find_devices();

	/* Search for the first device with the Cypress
	 * Vendor and Produce ID's. */

	numfound = 0;
	for(bus = usb_get_busses();bus ; bus = bus->next) {
    		struct usb_device *dev;
		for( dev = bus->devices ; dev ; dev = dev->next ) {
			if( dev->descriptor.idVendor == 0x0547 &&
			    dev->descriptor.idProduct == 0x2131 ) {
				if(numfound==0) fx2 = dev;
				numfound++;
			}
		}
	}

	if(numfound==0) {
		fprintf(stderr,"No Cypress CY7C68013A found.\n");
		fprintf(stderr,"Has the firmware been loaded?\n");
		exit(2);
	} else if(numfound!=1) {
		fprintf(stderr,"Multiple USB devices found. Using the first\n");
	}

	if( fx2 ) {
		struct usb_dev_handle *usbhandle;
		int n, k;
		int altif = 1;   /* Select the desired interface configuration. */
		usbhandle = usb_open( fx2 );
		n = usb_set_configuration(usbhandle,1);
		if(n<0) {
			fprintf(stderr,"usb_set_configuration() failed\n");
		}
		n = usb_claim_interface(usbhandle,0);
		if(n<0) {
			fprintf(stderr,"usb_claim_interface() failed\n");
			exit(2);  /* Can't do anything useful if this doesn't work. */
		}
		n = usb_set_altinterface(usbhandle,altif);
		if(n<0) {
			fprintf(stderr,"usb_set_altinterface(%d) failed\n",altif);
			fprintf(stderr,"Operation may be flakey\n");
		}

		/****************************
		 * Need to change stdin to non-blocking. */

		n = fcntl(0,F_GETFL);
		if(n<0) {
			fprintf(stderr,"Trouble getting stdin flags\n");
			n = 0;
		}
		n = fcntl(0,F_SETFL,n|O_NONBLOCK);
		if(n<0) {
			fprintf(stderr,"Trouble setting stdin flags\n");
			n = 0;
		}

		n = 0;
		k = 1;
		for(;;) {
			char outbuf[1024];
			char inbuf[1024];
			if(n==0) {
				n = read( 0, outbuf, 1024 );
				if(n<0) {
					if(errno == EAGAIN) {
						usleep(10000);
						n = 0;
					}
					else {
						fprintf(stderr,"read n= %d err = %d\n",n,errno);
					}
				}
			}
			if(n>0) {
				int rtn;
				rtn = usb_bulk_write(usbhandle,0x02,outbuf,n-1,10); 
				if(rtn >= 0) n = 0;
				fprintf(stderr,"usb_bulk_write %d\n",rtn);
			}
			if(k==0) {
				int rtn;
				rtn = usb_bulk_read(usbhandle,0x06,inbuf,1024,10); 
				if(rtn != -110) fprintf(stderr,"Got %d\n",rtn);
				if(rtn>0) k = rtn;
			}
			if(k>0) {
				fprintf(stdout,"usb read %d bytes\n",k);
				/* TODO: Print them. */
				k = 0;
			}
		}
		/* Currently, we never get here. */
		printf( "usb_close\n" );
		usb_close( usbhandle );
	}

	return 0;
}
